package ar.rulosoft.mimanganu.adapters;

import android.content.Context;
import android.support.v7.app.AppCompatActivity;
import android.view.LayoutInflater;
import android.view.View;
import android.view.View.OnClickListener;
import android.view.ViewGroup;
import android.widget.ArrayAdapter;
import android.widget.ImageButton;
import android.widget.ProgressBar;
import android.widget.TextView;

import java.util.ArrayList;

import ar.rulosoft.mimanganu.R;
import ar.rulosoft.mimanganu.services.ChapterDownload;
import ar.rulosoft.mimanganu.services.DownloadPoolService;

public class DownloadAdapter extends ArrayAdapter<ChapterDownload> {

    private static String[] states;
    private static int listItem = R.layout.listitem_descarga;
    private ArrayList<ChapterDownload> downloads = new ArrayList<>();
    private LayoutInflater li;
    private boolean darkTheme;
    private AppCompatActivity mActivity;

    public DownloadAdapter(Context context, AppCompatActivity activity, boolean darkTheme) {
        super(context, listItem);
        states = context.getResources().getStringArray(R.array.estados_descarga);
        li = (LayoutInflater) context.getSystemService(Context.LAYOUT_INFLATER_SERVICE);
        mActivity = activity;
        this.darkTheme = darkTheme;
    }

    @Override
    public ChapterDownload getItem(int position) {
        return downloads.get(position);
    }

    @Override
    public int getCount() {
        return downloads.size();
    }

    @Override
    public void add(ChapterDownload object) {
        downloads.add(object);
    }

    @Override
    public View getView(int position, View convertView, ViewGroup parent) {
        ViewHolder holder;
        if (convertView == null) {
            convertView = li.inflate(listItem, null);
            holder = new ViewHolder(convertView, darkTheme);
            convertView.setTag(holder);
        } else {
            holder = (ViewHolder) convertView.getTag();
        }

        final ChapterDownload item = getItem(position);

        if (item != null) {
            String textInfo = " " + states[item.status.ordinal()];
            holder.textViewName.setText(android.text.Html.fromHtml(item.getChapter().getTitle() + textInfo));
            holder.loadingProgressBar.setMax(item.getChapter().getPages());
            holder.loadingProgressBar.setProgress(item.getProgress());
            holder.buttonImageView.setOnClickListener(new OnClickListener() {
                @Override
                public void onClick(View v) {
                    if (DownloadPoolService.removeDownload(item.chapter.getId(), getContext())) {
                        remove(item);
                        mActivity.runOnUiThread(new Runnable() {
                            @Override
                            public void run() {
                                DownloadAdapter.this.notifyDataSetChanged();
                            }
                        });
                    }
                }
            });
        }
        return convertView;
    }

    @Override
    public void remove(ChapterDownload object) {
        downloads.remove(object);
    }

    public synchronized void updateAll(ArrayList<ChapterDownload> mDescargas) {
        if (mDescargas != null) {
            for (int i = 0; i < mDescargas.size(); i++) {
                boolean isNew = true;
                ChapterDownload toCompare = mDescargas.get(i);
                for (int j = 0; j < getCount(); j++) {
                    if (getItem(j).getChapter().getId() == toCompare.getChapter().getId()) {
                        isNew = false;
                        ChapterDownload item = getItem(j);
                        item.setProgress(toCompare.getProgress());
                        item.status = toCompare.status;
                    }
                }
                if (isNew) {
                    downloads.add(toCompare);
                }
            }
        }

    }

    public static class ViewHolder {
        private TextView textViewName;
        private ProgressBar loadingProgressBar;
        private ImageButton buttonImageView;

        public ViewHolder(View v, boolean darkTheme) {
            this.textViewName = (TextView) v.findViewById(R.id.nombre);
            this.buttonImageView = (ImageButton) v.findViewById(R.id.boton);
            this.loadingProgressBar = (ProgressBar) v.findViewById(R.id.progreso);
            if (darkTheme)
                this.buttonImageView.setImageResource(R.drawable.ic_action_x_dark);
        }
    }

}
